'
' isswcmPrepService.vbs
'
' Automatic start restoration tool for service
'
' Copyright 2010 FUJITSU LIMITED
'
Option Explicit


'Error code definition
Const ERROR_SV_SUCCESS			= 0
Const ERROR_SV_UNKNOWN			= 101
Const ERROR_SV_BADCOMMAND		= 102
Const ERROR_SV_BADENVIRONMENT	= 103
Const ERROR_SV_BADOPTION		= 104
Const ERROR_SV_TARGETCONFLICT	= 105
Const ERROR_SV_NOTARGET			= 106
Const ERROR_SV_NOGROUP			= 107
Const ERROR_SV_READNAMEFILE		= 108
Const ERROR_SV_READREGFILE		= 109
Const ERROR_SV_WRITEREGFILE		= 110
Const ERROR_SV_BADREGFILE		= 111
Const ERROR_SV_GROUPNOTFOUND	= 112
Const ERROR_SV_READSTATEFILE	= 113
Const ERROR_SV_WRITESTATEFILE	= 114
Const ERROR_SV_BADSTATEFILE		= 115
Const ERROR_SV_SUPPRESSSTARTUP	= 116
Const ERROR_SV_STOPSERVICE		= 117
Const ERROR_SV_NOSTATEFILE		= 118
Const ERROR_SV_RESTORESTARTUP	= 119
Const ERROR_SV_STARTSERVICE		= 120



Const strEnvLogFile		= "SCW_PREP_SERVICE_LOG"
Const strRegistFile		= "svcinfo.xml"
Const strDefStateFile	= "svcstate.txt"

' Default value at time until parents service is stopped after subordination service is stopped.
Const lngDefWait = 10

' Default value of frequency retrying
Const lngDefMaxRetry = 600


'The error occurred.
Sub RaiseError(num, desc)
	Dim strMsg
	strMsg = "ERROR(" & CStr(num) & "): " & desc
	Call Trace(strMsg)
	WScript.Echo(strMsg)
	Err.Raise vbObjectError + num, , desc
End Sub

' The error message of Win32_Service is gathered. (fo log)
Function GetServiceErrorMessage(ByVal lngError)
	Dim strRet
	Select Case lngError
		Case 0		strRet = "Success"
		Case 1		strRet = "Not Supported"
		Case 2		strRet = "Access Denied"
		Case 3		strRet = "Dependent Services Running"
		Case 4		strRet = "Invalid Service Control"
		Case 5		strRet = "Service Cannot Accept Control"
		Case 6		strRet = "Service Not Active"
		Case 7		strRet = "Service Request Timeout"
		Case 8		strRet = "Unknown Failure"
		Case 9		strRet = "Path Not Found"
		Case 10		strRet = "Service Already Running/Stopped"
		Case 11		strRet = "Service Database Locked"
		Case 12		strRet = "Service Dependency Deleted"
		Case 13		strRet = "Service Dependency Failure"
		Case 14		strRet = "Service Disabled"
		Case 15		strRet = "Service Logon Failure"
		Case 16		strRet = "Service Marked For Deletion"
		Case 17		strRet = "Service No Thread"
		Case 18		strRet = "Status Circular Dependency"
		Case 19		strRet = "Status Duplicate Name"
		Case 20		strRet = "Status Invalid Name"
		Case 21		strRet = "Status Invalid Parameter"
		Case 22		strRet = "Status Invalid Service Account"
		Case 23		strRet = "Status Service Exists"
		Case 24		strRet = "Service Already Paused"
		Case Else	strRet = "Unknown Error Code"
	End Select
	GetServiceErrorMessage = strRet
End Function

' I display the usage. 
Sub Usage
	Dim strMsg
	strMsg = "Usage:" & vbCrLf &_
		"cscript.exe //nologo prepservice.vbs <subcommand> [options...]" & vbCrLf & vbCrLf &_
		"Subcommands:" & vbCrLf & vbCrLf &_
		"  StartServicesBasedOnOSConf [ -O:File ] [ -R:Count ] [ -C:Category ]" & vbCrLf &_
		"    Restores running state of the registered services." & vbCrLf
	WScript.Echo(strMsg)
End Sub


' Log output
Sub Trace(strLog)
	Dim objFSO, objShell, objEnv, objLogFile, strLogFile
	Set objShell = WScript.CreateObject("WScript.Shell")
	Set objEnv = objShell.Environment("Process")
	strLogFile = objEnv(strEnvLogFile)

'	WScript.Echo(strLog)
	If Len(strLogFile) > 0 Then
		Set objFSO = WScript.CreateObject("Scripting.FileSystemObject")
		Set objLogFile = objFSO.OpenTextFile(strLogFile, 8, True)
		objLogFile.WriteLine(Now & " " & strLog)
		objLogFile.Close
	End If
End Sub

' escape the character that cannot be used for XML. 
Function EscapeString(ByVal strText)
	strText = Replace(strText, "&", "&amp;")
	strText = Replace(strText, """", "&quot;")
	strText = Replace(strText, "'", "&apos;")
	strText = Replace(strText, "<", "&lt;")
	strText = Replace(strText, ">", "&gt;")
	EscapeString = strText
End Function


' return it based on the character where it escapes. 
Function UnescapeString(ByVal strText)
	strText = Replace(strText, "&gt;", ">")
	strText = Replace(strText, "&lt;", "<")
	strText = Replace(strText, "&apos;", "'")
	strText = Replace(strText, "&quot;", """")
	strText = Replace(strText, "&amp;", "&")
	UnescapeString = strText
End Function


' preserve XML in the file. 
Function SaveXML(objXML, strXMLFile)
	Dim lngTmpCode, strTmpDesc
	On Error Resume Next
	Err.Clear
	Call objXML.save(strXMLFile)
	If Err = 0 Then
		SaveXML = ""
	Else
		SaveXML = Err.Description & "(SubCode=" & CStr(Err.Number) & ")"
	End If
End Function

' The registered service is read from the file, and it puts it in the array. 
Sub LoadRegisteredServices(arrSvcNames, strCategory)
	Call Trace("LoadRegisteredServices")
	Dim strRegistFilePath, strScript, nLastBS, bRet
	Dim strPattern
	Dim objXML, objFSO
	Dim objRoot, objSvcList, objService
	Dim I, strSvcName, strTmp, bFound
        Dim objShell, objEnv
	
	Redim arrSvcNames(-1)	' All deletions

	Set objShell = WScript.CreateObject("WScript.Shell")
        Set objEnv = objShell.Environment("Process")
'	strScript = WScript.ScriptFullName
'	nLastBS = InStrRev(strScript, "\")
'	strRegistFilePath = Left(strScript, nLastBS) & strRegistFile
	strRegistFilePath = objEnv.Item("INST_DIR") & "\bin\" & strRegistFile

	Set objXML = WScript.CreateObject("MSXML.DOMDocument")
	objXML.async = False

	Set objFSO = WScript.CreateObject("Scripting.FileSystemObject")
	If Not objFSO.FileExists(strRegistFilePath) Then
		' Service is unregistered because there is no registration file.
		' no object application program. not error. 
		Call Trace("Registration file does not exists, skipped")
		Exit Sub
	Else
		' Route node acquisition
		Call Trace("Loading registration file: " & strRegistFilePath)
		bRet = objXML.load(strRegistFilePath)
		If Not bRet Then
			Call RaiseError(ERROR_SV_READREGFILE, "Failed to load registered services (bad format)")
		End If
		Set objRoot = objXML.documentElement
		Call Trace("Loading completed")
	End If

	' All services are acquired regardless of the belonging group.
	If Len(strCategory) > 0 Then
		strPattern = "//Service[@Category=""" & EscapeString(strCategory) & """]"
	Else
		strPattern = "//Service[not(@Category)]"
	End If
	Set objSvcList = objRoot.selectNodes(strPattern)
	Call Trace(CStr(objSvcList.length) & " services are found in the category """ & strCategory & """")
	For I=0 To objSvcList.length - 1
		Set objService = objSvcList.nextNode
		strSvcName = UnescapeString(objService.getAttribute("Name"))

		' adds it to the array if it has not acquired it. 
		bFound = False
		For Each strTmp In arrSvcNames
			If strTmp = strSvcName Then
				bFound = True
			End If
		Next
		If Not bFound Then
			ReDim Preserve arrSvcNames(UBound(arrSvcNames) + 1)
			arrSvcNames(UBound(arrSvcNames)) = strSvcName
		End If
	Next
End Sub

Sub DoStartServicesWithOSConf(strStateFile, arrSvcNames, byVal lngMaxRetry)
	Dim objXML, objFSO, bRet
	Dim objRoot, objGroup, objService
	Dim intRetryCount
    Dim objShell, objEnv, strLogFilePath, dasStart

	Set objXML = WScript.CreateObject("MSXML.DOMDocument")
	objXML.async = False

	Set objFSO = WScript.CreateObject("Scripting.FileSystemObject")
	If Not objFSO.FileExists(strStateFile) Then
		' Error. The restoration is impossible without preserving it. 
		Call RaiseError(ERROR_SV_NOSTATEFILE, "State file is not exists: " & strStateFile)
	End If
    Set objShell = WScript.CreateObject("WScript.Shell")
    Set objEnv = objShell.Environment("Process")
    strLogFilePath = objEnv.Item(strEnvLogFile)
    dasStart = """" & objEnv.Item("INST_DIR") & "\F3FMisjee\bin\ijdasstart"""
	
	' The status preservation file is read. 
	Call Trace("Loading state file: " & strStateFile)
	bRet = objXML.load(strStateFile)
	If Not bRet Then
		Call RaiseError(ERROR_SV_READSTATEFILE, "Failed to load state file (bad format)")
	End If
	Set objRoot = objXML.documentElement
	Call Trace("Loading completed")

	Call Trace("Starting the services")
	Dim objWMI
	Set objWMI = GetObject("winmgmts:\\.\root\cimv2")

	Dim strSvcName, strStartup, strRunning
	Dim objSvcs, objSvc
	Dim strMode, intRet

	For Each strSvcName In arrSvcNames
		Set objSvcs = objWMI.ExecQuery("Select * from Win32_Service where DisplayName='" &_
			strSvcName & "'")
		If objSvcs.Count = 0 Then
			Call Trace("Service """ & strSvcName & """ was not found on this machine, skipped")
		End If

		For Each objSvc In objSvcs	' It is sure to become a hit only by 1 piece. 
			Set objService = objRoot.selectSingleNode("./Service[@Name=""" &_
				EscapeString(strSvcName) & """ and @Running]")
			' When there is only information
			' Restoration of running state
			' When the start setting is not only "Invalidity" when not being executing it, it starts. 
			If Not objSvc.Started And objSvc.StartMode = "Auto" Then
                If strSvcName = "Interstage Java EE DAS" Then
                    Call Trace("Starting """ & strSvcName & """")
                    Call Trace(dasStart)
                    intRet = objShell.Run("cmd.exe /c " & dasStart & " >> " & strLogFilePath & " 2>&1", 0, True)
                    If intRet = 0 Then
                        Call Trace("Started(" & CStr(intRet) & ")")
                    Else
                        Call RaiseError(ERROR_SV_STARTSERVICE, "StartService failed(" & CStr(intRet) & ")")
                    End If
                Else
					For intRetryCount=0 To lngMaxRetry
						Call Trace("Starting """ & strSvcName & """")
						intRet = objSvc.StartService
						If intRet = 0 Then
							Call Trace("Started(" & CStr(intRet) & ")")
							Exit For
						ElseIf intRet = 11 Then
							' Retrying when service DB is locked
							Call Trace("StartService failed (Service database locked), Retry...")
						Else
							' Other errors
							Call RaiseError(ERROR_SV_STARTSERVICE, "StartService failed: " &_
								GetServiceErrorMessage(intRet) & " (" & CStr(intRet) & ")")
						End If
						WScript.Sleep(1000)
					Next

					If intRet <> 0 Then
						' It did not succeed in the frequency retrying. 
						Call RaiseError(ERROR_SV_STARTSERVICE, "Retry count exceeded")
					End If
                End If
			End If
            If Not objService Is Nothing Then
				' delete
				Call objRoot.removeChild(objService)
			Else
				' It is not preserved. 
				Call Trace("Service """ & strSvcName & """ has not been saved or already restored... skipped")
			End If
		Next
	Next
	Call Trace("Starting the services has completed")

	' Status preservation file update
	Dim strSave
	Call Trace("Saving state file")
	strSave = SaveXML(objXML, strStateFile)
	If strSave <> "" Then
		Call RaiseError(ERROR_SV_WRITESTATEFILE, "Failed to save state file: " & strSave)
	End If
	Call Trace("Saving completed")
End Sub

' Analysis of command of restart
Sub StartServicesWithOSConf(objArgs)
	Call Trace("Function: StartServices")
	Dim lngMaxRetry, strStateFile, I
	Dim strScript, nLastBS
	Dim strCategory
	Dim arrSvcNames()
        Dim objShell, objEnv
	
	lngMaxRetry = lngDefMaxRetry

	Set objShell = WScript.CreateObject("WScript.Shell")
        Set objEnv = objShell.Environment("Process")
'	strScript = WScript.ScriptFullName
'	nLastBS = InStrRev(strScript, "\")
'	strStateFile = Left(strScript, nLastBS) & strDefStateFile
        strStateFile = objEnv.Item("INST_DIR") & "\bin\" & strDefStateFile

	For I=1 To objArgs.Count - 1
		Select Case Left(objArgs(I), 2)
			Case "-O"
				If Len(objArgs(I)) < 4 Or Mid(objArgs(I), 3, 1) <> ":" Then
					Call RaiseError(ERROR_SV_BADOPTION, "Invalid option: """ & objArgs(I) & """")
				End If
				strStateFile = Mid(objArgs(I), 4)
				
			Case "-R"
				If Len(objArgs(I)) < 4 Or Mid(objArgs(I), 3, 1) <> ":" Then
					Call RaiseError(ERROR_SV_BADOPTION, "Invalid option: """ & objArgs(I) & """")
				End If
				On Error Resume Next
				lngMaxRetry = CLng(Mid(objArgs(I), 4))
				If Err <> 0 Then lngMaxRetry = -1
				On Error Goto 0
				If lngMaxRetry < 0 Then
					Call RaiseError(ERROR_SV_BADOPTION, "Invalid option: """ & objArgs(I) & """")
				End If
				
			Case "-C"	' specify the category name
				If Len(objArgs(I)) < 4 Or Mid(objArgs(I), 3, 1) <> ":" Then
					Call RaiseError(ERROR_SV_BADOPTION, "Invalid option: """ & objArgs(I) & """")
				End If
				strCategory = Mid(objArgs(I), 4)

			Case Else
				Call RaiseError(ERROR_SV_BADOPTION, "Invalid option: """ & objArgs(I) & """")
		End Select
	Next

	Call LoadRegisteredServices(arrSvcNames, strCategory)

	Call Trace("state file: " & strStateFile)
	Dim strSvc
	For Each strSvc In arrSvcNames
		Call Trace("target: " & strSvc)
	Next

	Call DoStartServicesWithOSConf(strStateFile, arrSvcNames, lngMaxRetry)
End Sub

Sub Main(objArgs)
	' After the usage is displayed, it ends if there is no argument. 
	If objArgs.Count = 0 Then
		Call Usage
		WScript.Quit(0)
	End If

	Call Trace("--- START ---")
	' Operation is specified by the first argument.
	Dim strSubCmd
	strSubCmd = LCase(objArgs(0))
	Select Case strSubCmd
		Case "startserviceswithosconf"
			Call StartServicesWithOSConf(objArgs)
		Case Else
			Call RaiseError(ERROR_SV_BADCOMMAND, "Invalid command: """ & strSubCmd & """")
	End Select
End Sub



On Error Resume Next
Dim nCode, strMsg

Call Main(WScript.Arguments)
If Err < 0 Then
	' Error that generates it with RaiseError
	nCode = Err.Number - vbObjectError
ElseIf Err > 0 Then
	' Error when VBScript is executed

	strMsg = "RUNTIME ERROR(" & Err.Number & "): " & Err.Description
	Call Trace(strMsg)
	WScript.Echo(strMsg)
	nCode = Err.Number

Else
	' success
	Call Trace("Succeeded")
	nCode = 0
End If

WScript.Quit(nCode)
